use std::path::PathBuf;

use serde::{Deserialize, Serialize};

#[cfg(not(build_only))]
use ts_rs::TS;

#[cfg(build_only)]
use crate::poe_reader;

#[cfg_attr(not(build_only), derive(TS))]
#[cfg_attr(not(build_only), ts(rename_all = "camelCase"))]
#[derive(Serialize, Deserialize, Debug, Default, Clone)]
#[serde(rename_all = "camelCase")]
pub struct Rect {
    x: f32,
    y: f32,
    width: f32,
    height: f32,
}

#[cfg_attr(not(build_only), derive(TS))]
#[cfg_attr(not(build_only), ts(rename_all = "camelCase"))]
#[derive(Serialize, Deserialize, Debug, Clone)]
#[serde(rename_all = "camelCase")]
pub struct Config {
    #[serde(default = "Config::default_initial_plan_window_position")]
    pub initial_plan_window_position: Rect,
    #[serde(default = "Config::default_initial_agg_window_position")]
    pub initial_agg_window_position: Rect,
    #[serde(default = "Config::default_initial_notes_window_position")]
    pub initial_notes_window_position: Rect,
    #[serde(default = "Config::default_hide_on_unfocus")]
    pub hide_on_unfocus: bool,
    #[serde(default = "Config::default_toggle_overlay")]
    pub toggle_overlay: String,
    #[serde(default = "Config::default_prev")]
    pub prev: String,
    #[serde(default = "Config::default_next")]
    pub next: String,
    #[serde(default = "Config::default_plan_bg")]
    pub plan_bg: String,
    #[serde(default = "Config::default_backdrop_bg")]
    pub backdrop_bg: String,
    #[serde(default = "Config::default_note_default_fg")]
    pub note_default_fg: String,
    #[serde(default = "Config::default_poe_client_log_path")]
    pub poe_client_log_path: Option<PathBuf>,

    #[serde(default = "Config::default_plan_num_visible")]
    pub num_visible: u16,
    #[serde(default = "Config::default_plan_offset")]
    pub offset: i16,

    #[serde(default = "Config::default_enable_stopwatch")]
    pub enable_stopwatch: bool,

    #[serde(default = "Config::default_run_compare_history")]
    pub run_compare_history: Option<String>,
    #[serde(default = "Config::default_detach_notes")]
    detach_notes: bool,
    
    #[serde(default = "Config::default_show_livesplit")]
    show_livesplit: bool,
    #[serde(default = "Config::default_show_live_aggregate")]
    show_live_aggregate: bool,
    #[serde(default = "Config::default_shorten_zone_names")]
    shorten_zone_names: bool,
}

impl Default for Config {
    fn default() -> Self {
        Self {
            initial_plan_window_position: Self::default_initial_plan_window_position(),
            initial_agg_window_position: Self::default_initial_agg_window_position(),
            initial_notes_window_position: Self::default_initial_agg_window_position(),
            hide_on_unfocus: Self::default_hide_on_unfocus(),
            toggle_overlay: Self::default_toggle_overlay(),
            prev: Self::default_prev(),
            next: Self::default_next(),
            plan_bg: Self::default_plan_bg(),
            backdrop_bg: Self::default_backdrop_bg(),
            #[cfg(build_only)]
            poe_client_log_path: Self::default_poe_client_log_path(),
            #[cfg(not(build_only))]
            poe_client_log_path: Self::default_poe_client_log_path(),
            note_default_fg: Self::default_note_default_fg(),
            num_visible: Self::default_plan_num_visible(),
            offset: Self::default_plan_offset(),
            run_compare_history: Self::default_run_compare_history(),
            detach_notes: Self::default_detach_notes(),
            enable_stopwatch: Self::default_enable_stopwatch(),
            show_livesplit: Self::default_show_livesplit(),
            show_live_aggregate: Self::default_show_live_aggregate(),
            shorten_zone_names: Self::default_shorten_zone_names(),
        }
    }
}

impl Config {
    fn default_initial_plan_window_position() -> Rect {
        Default::default()
    }

    fn default_initial_agg_window_position() -> Rect {
        Default::default()
    }

    fn default_initial_notes_window_position() -> Rect {
        Default::default()
    }

    fn default_hide_on_unfocus() -> bool {
        true
    }

    fn default_toggle_overlay() -> String {
        "F6".into()
    }

    fn default_prev() -> String {
        "ArrowLeft".into()
    }

    fn default_next() -> String {
        "ArrowRight".into()
    }

    fn default_plan_bg() -> String {
        "#00000010".to_string()
    }

    fn default_backdrop_bg() -> String {
        "#00000030".to_string()
    }

    #[cfg(build_only)]
    fn default_poe_client_log_path() -> Option<PathBuf> {
        poe_reader::get_poe_path()
    }
    #[cfg(not(build_only))]
    fn default_poe_client_log_path() -> Option<PathBuf> {
        None
    }

    fn default_note_default_fg() -> String {
        "#bfbfbfff".to_string()
    }

    fn default_plan_num_visible() -> u16 {
        3
    }
    fn default_plan_offset() -> i16 {
        1
    }
    fn default_enable_stopwatch() -> bool {
        false
    }
    
    fn default_run_compare_history() -> Option<String> {
        None
    }
    fn default_show_livesplit() -> bool {
        false
    }
    fn default_show_live_aggregate() -> bool {
        false
    }
    fn default_shorten_zone_names() -> bool {
        false
    }
    fn default_detach_notes() -> bool {
        false
    }
}
