import { AfterViewInit, Component, ElementRef, Input, ViewChild } from '@angular/core';
import { AngularResizeEventModule, ResizedEvent } from 'angular-resize-event';

@Component({
  selector: 'scalable',
  templateUrl: './scalable.component.html',
  imports: [ AngularResizeEventModule ],
  standalone: true
})
export class ScalableComponent implements AfterViewInit {
  @ViewChild('container') container?: ElementRef;
  @ViewChild('element') element?: ElementRef;

  @Input() clamp?: number;

  ngAfterViewInit(): void {
    if (!this.container || !this.element) {
      return;
    }

    setTimeout(() => this.update());
  }

  update() {
    if(!this.container || !this.element) {
      return;
    }

    const containerWidth = this.container?.nativeElement.clientWidth;
    const containerHeight = this.container?.nativeElement.clientHeight;
    const elementWidth = this.element?.nativeElement.clientWidth;
    const elementHeight = this.element?.nativeElement.clientHeight;

    this.element.nativeElement.style.transform = "";
    
    this.element.nativeElement.style.left = '0';
    this.element.nativeElement.style.top = '0';

    this.element.nativeElement.style.transformOrigin = "left top";

    let scale = containerWidth / elementWidth;

    if(containerHeight < elementHeight * scale) {
      scale = containerHeight / elementHeight;
    }
    if(this.clamp) {
      scale = Math.min(this.clamp, scale);
    }

    this.element.nativeElement.style.transform += `scale(${scale}, ${scale})`;
  }

  onResize(_event: ResizedEvent) {
    this.update();
  }
}
