import { AfterContentInit, AfterViewInit, Component, ElementRef, EventEmitter, Input, OnInit, Output, ViewChild } from '@angular/core';
import Picker from 'vanilla-picker';
import { MatButtonModule } from '@angular/material/button';
import { Subject, delay, startWith } from 'rxjs';
import { CommonModule } from '@angular/common';

export interface Color {
  rgba: number[];
  hsla: number[];
  rgbString: string;
  rgbaString: string;
  hslString: string;
  hslaString: string;
  hex: string;
}

export interface Placement {
  color: 'top' | 'bottom' | 'left' | 'right' | false;
}

@Component({
  selector: 'color-picker',
  templateUrl: './color-picker.component.html',
  styleUrls: ['./color-picker.component.scss'],
  imports: [
    CommonModule,
    MatButtonModule
  ],
  standalone: true
})
export class ColorPickerComponent implements AfterViewInit {

  @ViewChild('clickColorPickerElement')
  clickColorPickerElement?: ElementRef;

  @Output('color')
  outColor = new EventEmitter<Color>;

  underlyingValue?: string;

  justSet: boolean = false;

  @Input() placement: Placement = { color: 'right'};

  @Input('setColor') set color(value: string) {
    this.justSet = true;
    if(this.picker) {
      this.picker.setColor(value, false);
    } 
    this.initialColor = value;
  }

  get value () {
    if(this.initialColor) {
      return this.initialColor;
    }
    if(this.underlyingValue) {
      return this.underlyingValue;
    }
    return undefined;
  }

  initialColor?: string;

  loaded: Subject<void> = new Subject<void>();

  picker?: Picker;
  showPicker: boolean = false;

  first: boolean = true;

  constructor() {
    this.loaded.pipe(delay(0)).subscribe(() => {
      this.picker = new Picker({
        onChange: (color: Color) => {
          if (this.justSet) {
            this.justSet = false;
            return;
          }
          this.underlyingValue = color.hex;
          return this.outColor.next(color);
        },
        parent: this.clickColorPickerElement?.nativeElement,
        color: this.initialColor,
        popup: this.placement.color,
      });
    });
    this.underlyingValue = this.color;
  }

  ngAfterViewInit(): void {
    this.loaded.next();
  }
}
