import { AfterViewInit, ChangeDetectorRef, Component, EventEmitter, Input, NgZone, OnInit, Output, ViewChild } from '@angular/core';
import { Rect } from '../_models/generated/Rect';
import { NgxMoveableComponent, NgxMoveableModule, OnDrag, OnDragEnd, OnResize, OnResizeEnd } from 'ngx-moveable';
import { CommonModule } from '@angular/common';

@Component({
  selector: 'app-draggable-window',
  templateUrl: './draggable-window.component.html',
  styleUrls: ['./draggable-window.component.scss'],
  standalone: true,
  imports: [CommonModule, NgxMoveableModule]

})
export class DraggableWindowComponent implements AfterViewInit {
  rect?: Rect;
  bounds: any = { "left": 0, "top": 0, "right": 0, "bottom": 0, "position": "css" };
  @Input() public backgroundColor: string = "rgba(0, 0, 0, 0.1)";
  @Input() public configurable: boolean = true;
  @Input() public initialRect?: Rect;
  @Output() savedRect: EventEmitter<Rect> = new EventEmitter();
  @ViewChild("moveable") moveable?: NgxMoveableComponent;
  init = false;


  constructor(private cdr: ChangeDetectorRef, private zone: NgZone) {
    window.addEventListener("resize", () => {
      this.zone.run(() => {
        this.windowInitHandler()
      })
    });
  }

  windowInitHandler() {
    if (window.innerWidth > 0) {
      const cfgRect = this.initialRect || { x: 0, y: 0, width: 0.1, height: 0.1 };

      this.rect = {
        x: cfgRect.x * window.innerWidth,
        y: cfgRect.y * window.innerHeight,
        width: cfgRect.width * window.innerWidth,
        height: cfgRect.height * window.innerHeight,
      }

      this.moveable?.updateRect();

      setTimeout(() => this.cdr.detectChanges(), 0);
      this.init = true;
    }
  }

  ngAfterViewInit(): void {
    this.windowInitHandler();
  }

  transform() {
    return `translate(${this.rect!.x}px, ${this.rect!.y}px)`;
  }

  onDrag(e: OnDrag) {
    this.rect!.x = e.translate[0];
    this.rect!.y = e.translate[1];
  }

  onDragEnd(_e: OnDragEnd) {
    this.saveRect();
  }

  onResize(e: OnResize) {
    this.rect!.width = e.width;
    this.rect!.height = e.height;
    this.onDrag(e.drag);
  }

  onResizeEnd(_e: OnResizeEnd) {
    this.saveRect();
  }

  saveRect() {
    if (this.rect) {
      this.savedRect.emit({
        x: this.rect.x / window.innerWidth,
        y: this.rect.y / window.innerHeight,
        width: this.rect.width / window.innerWidth,
        height: this.rect.height / window.innerHeight,
      });
    }
  }
}
