import {  Component, ElementRef, Inject, Input,   ViewChild, } from '@angular/core';
import { MAT_DIALOG_DATA, MatDialogRef, MatDialogModule } from '@angular/material/dialog';
import { CommonModule } from '@angular/common';
import { MatButtonModule } from '@angular/material/button';
import { FormsModule } from '@angular/forms';
import { MatInputModule } from '@angular/material/input';
import { MatFormFieldModule } from '@angular/material/form-field';
import { MarkdownService } from 'src/app/_services/markdown.service';
import { ScalableComponent } from 'src/app/Scalable/scalable.component';


interface DialogData {
  note?: string;
}

@Component({
  selector: 'notes',
  templateUrl: './notes.component.html',
  styleUrls: ['./notes.component.scss'],
  standalone: true,
  imports: [CommonModule, FormsModule, MatButtonModule],
})
export class NotesComponent {
  @Input()
  note?: string;

  @ViewChild("ref")
  ref?: ElementRef

  realValue: any;

  constructor(public md: MarkdownService) {}  
}

@Component({
  selector: 'notes-editor',
  templateUrl: 'edit-notes.component.html',
  styleUrls: ['./edit-notes.component.scss'],
  standalone: true,
  imports: [CommonModule, FormsModule, MatDialogModule, MatFormFieldModule, MatInputModule, FormsModule, MatButtonModule, NotesComponent, ScalableComponent],
})
export class EditNotesComponentDialog {
  note?: string;

  constructor(
    public dialogRef: MatDialogRef<EditNotesComponentDialog>,
    @Inject(MAT_DIALOG_DATA) public data: DialogData,
    public md: MarkdownService
  ) {
    if (data.note) {
      this.note = `${data.note}`;
    } else {
      this.note = "";
    }
  }

  cancel() {
    this.dialogRef.close(undefined);
  }

  save() {
    this.dialogRef.close(this.note);
  }
}
