import { Component, Inject, Input, NgZone } from '@angular/core';
import { CommonModule } from '@angular/common';
import { FormsModule } from '@angular/forms';
import { ConfigService } from '../_services/config.service';
import { Color, ColorPickerComponent } from '../color-picker/color-picker.component';
import {MatSlideToggleModule} from '@angular/material/slide-toggle';
import { OverlayService } from '../_services/overlay.service';
import { PlanService } from '../_services/plan.service';
import { MAT_DIALOG_DATA, MatDialog, MatDialogModule, MatDialogRef } from '@angular/material/dialog';
import { MatFormFieldModule } from '@angular/material/form-field';
import { MatInputModule } from '@angular/material/input';
import { MatButtonModule } from '@angular/material/button';
import { ShortcutService } from '../_services/shortcut.service';
import { RecordKeyChord } from '../directives/record-key-chord.directive';
import { MatDividerModule } from '@angular/material/divider';

@Component({
  selector: 'settings',
  standalone: true,
  imports: [CommonModule, FormsModule, ColorPickerComponent, MatSlideToggleModule, MatDialogModule, MatButtonModule, MatDividerModule, MatFormFieldModule, MatInputModule],
  templateUrl: './settings.component.html',
  styleUrls: ['./settings.component.scss']
})
export class SettingsComponent {
  @Input() 
  overlay: boolean = false;
  
  constructor(
    public configService: ConfigService,
    private zone: NgZone,
    public overlayService: OverlayService,
    private planService: PlanService,
    public dialog: MatDialog,
    private shortcut: ShortcutService
  ) {

  }

  onPlanColorChange(color: Color) {
    this.zone.run(() => this.configService.config.planBg = color.rgbaString);

  }

  onBackdropColorChange(color: Color) {
    this.zone.run(() => this.configService.config.backdropBg = color.rgbaString);
  }

  onNoteColorChange(color: Color) {
    this.zone.run(() => this.configService.config.noteDefaultFg = color.rgbaString);
  }

  

  rebindOverlayToggle() {
    const dialogRef = this.dialog.open(BindDialog, {
      data: {chord: this.configService.config.toggleOverlay},
    });

    dialogRef.afterClosed().subscribe(result => {
      if(result) {
        this.shortcut.rebind_from_to(this.configService.config.toggleOverlay, result);
        this.configService.config.toggleOverlay = result;
      }
    });
  }

  rebindPreviousZone() {
    const dialogRef = this.dialog.open(BindDialog, {
      data: {chord: this.configService.config.prev},
    });

    dialogRef.afterClosed().subscribe(result => {
      if(result) {
        this.shortcut.rebind_from_to(this.configService.config.prev, result);
        this.configService.config.prev = result;
      }
    });
  }

  rebindNextZone() {
    const dialogRef = this.dialog.open(BindDialog, {
      data: {chord: this.configService.config.next},
    });

    dialogRef.afterClosed().subscribe(result => {
      if(result) {
        this.shortcut.rebind_from_to(this.configService.config.next, result);
        this.configService.config.next = result;
      }
    });
  }

}

interface BindData {
  chord: string;
}

@Component({
  selector: 'bind-dialog',
  templateUrl: 'bind-dialog.html',
  standalone: true,
  imports: [MatDialogModule, MatFormFieldModule, MatInputModule, FormsModule, MatButtonModule, RecordKeyChord],
})
export class BindDialog {

  constructor(
    public dialogRef: MatDialogRef<BindDialog>,
    @Inject(MAT_DIALOG_DATA) public data: BindData,
  ) {}

  onNoClick(): void {
    this.dialogRef.close();
  }

  onChord(chord: string[]) {
    this.data.chord = chord.reduce((acc, curr) => {
      if (acc === '') return curr;

      return acc.concat('+').concat(curr);
    }, '');
  }
}